/* The zlib/libpng License

Copyright (c) 2005 Dirk Jagdmann <doj@cubic.org>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this
   software in a product, an acknowledgment in the product
   documentation would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
   distribution.
*/

#include "color.hpp"

double highthreshold=0.9;
double fontsize=15;

double cm2pt(double cm)
{
  return cm/2.54*72.0;
}

double pt2cm(double pt)
{
  return pt/72.0*2.54;
}

void setupFont(const string& name)
{
  cout << "/" << name << " findfont " << fontsize << " scalefont setfont\n";
}

ostream& print(rgba c)
{
  if(c.r<0.01) c.r=0;
  if(c.g<0.01) c.g=0;
  if(c.b<0.01) c.b=0;
  if(c.r>1.0) c.r=1;
  if(c.g>1.0) c.g=1;
  if(c.b>1.0) c.b=1;
  return cout << c.r << " " << c.g << " " << c.b;
}

ostream& setcol(rgba c)
{
  return print(c) <<  " setrgbcolor ";
}

ostream& print(double c)
{
#if 1
  if(c<0.05)
    c=0;
  if(c>highthreshold)
    c=1;
#endif
  return cout << c;
}

ostream& setcol(double c)
{
  return print(c) << " setgray ";
}

enum Textflags
  {
    Left = 0,
    Center = 1,
    Right = 2,
    Up = 4,
    Mid = 8,
    Stroke = 16,
  };

ostream& print(const string& text, double x=0, double y=0, rgba col=rgba(0,0,0), int flags=Left|Stroke)
{
  gray g; g=col;
  string t="("; t+=text+")";

  cout << "/textxpos " << x << " def\n";
  cout << "/textypos " << y << " def\n";
  cout << t << " stringwidth pop /textwidth exch def\n";
  cout << "/textheight " << fontsize << " def\n";

  if(flags&Center)
    cout << "/textxpos textxpos textwidth 2 div sub def\n";
  if(flags&Right)
    cout << "/textxpos textxpos textwidth sub def\n";

  if(flags&Mid)
    cout << "/textypos textypos textheight 3 div sub def\n";
  if(flags&Up)
    cout << "/textypos textypos textheight sub def\n";

  cout << "gsave\n";
  cout << 0.2*fontsize << " setlinewidth\n";
  cout << "textxpos textypos moveto\n";
  setcol((g.g>0.5)?0.0:1.0);
  cout << t << " true charpath stroke\n";
  cout << "grestore\n";

  cout << "gsave\n";
  cout << "textxpos textypos moveto\n";
  setcol(col);
  cout << t << " show\n";

  cout << "grestore ";
  return cout;
}
